/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DataBaseEventExtractor.java 7402 2011-01-06 09:49:25Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring;

import java.io.IOException;
import java.text.ParseException;
import java.util.Collection;
import java.util.Date;
import java.util.Hashtable;

import javax.naming.NamingException;

import org.ow2.jasmine.event.beans.JasmineEventEB;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventSLBRemote;
import org.ow2.jasmine.monitoring.eventswitch.connectors.EJB3Connector;

/**
 * Class for the extraction of the data form a database
 * @author Christophe Merisier
 */
public class DataBaseEventExtractor extends EJB3Connector<JasmineEventSLBRemote> {

    public DataBaseEventExtractor() {
        super();
    }

    public JasmineEventEB[] getEvents(
        final Collection<String> servers,
        final Collection<String> probes,
        final Date from,
        final Date to) throws NamingException, ParseException, IOException {

        // set configuration needed to connect to the bean
        Hashtable<String, String> options = new Hashtable<String, String>();
        options.put("Context.INITIAL_CONTEXT_FACTORY", "org.ow2.carol.jndi.spi.MultiOrbInitialContextFactory");
        options.put("Wrapper.LOOKUP_BEAN", "db-ejb/event");
        setConfiguration(options);

        // ask the bean to extract events from the database
        JasmineEventEB[] events = getBean().getEvents(servers, probes, from, to, "timestamp", 0, 300000);
        disconnect();

        return events;
    }
}
