/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DatabaseExtractor.java 7834 2011-03-17 15:13:37Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.service;

import org.ow2.jasmine.event.beans.JasmineEventEB;
import org.ow2.jasmine.monitoring.eos.monitoring.DataBaseEventExtractor;
import org.ow2.jasmine.monitoring.eos.monitoring.EventData;
import org.ow2.jasmine.monitoring.eos.monitoring.Serie;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * Extractor for Eventswitch database
 * @author durieuxp
 */
public class DatabaseExtractor extends Extractor {

    private static SimpleDateFormat format;

    // ------------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------------

    /**
     * Constructor for DashboardDataCollector
     */
    public DatabaseExtractor(DashboardDataCollector dbc, Date from, Date to) {
        super(dbc, from, to);
        format = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
    }

    // ------------------------------------------------------------------------
    // Thread implementation
    // ------------------------------------------------------------------------

    /**
     * Run method called when the thread is started.
     */
    public void run() {

        // Use a Set to avoid duplicate elements
        Set<String> servers = new HashSet<String>();
        Set<String> probes = new HashSet<String>();
        for (Serie element : dbc.getSeries()) {
            probes.add(element.getYcol());
            if (element.getServer() != null) {
                servers.add(element.getServer());
            }
        }

        // Connect to EJB and get the events from database
        DataBaseEventExtractor dataBaseEventExtractor = new DataBaseEventExtractor();
        JasmineEventEB[] events = null;
        try {
            events = dataBaseEventExtractor.getEvents(servers, probes, from, to);
        } catch (Exception e) {
            logger.error("Cannot read database: " + e);
            dbc.setReplayCompleted();
            return;
        }
        if (events == null) {
            logger.debug("Nothing read from database");
            dbc.setReplayCompleted();
            return;
        }

        // return only events matching one of the Series
        for (JasmineEventEB elem : events) {

            // Find series accepting this event.
            for (Serie curser : dbc.getSeries()) {

                // Check Attribute
                if (!elem.getProbe().endsWith(":" + curser.getYcol())) {
                    continue;
                }

                // Check pattern
                if (! dbc.match(elem.getMBean(), curser.getPattern())) {
                    continue;
                }

                // Check server
                if (curser.getServer() != null && curser.getServer().length() > 0 && !curser.getServer().equals(elem.getServer())) {
                    continue;
                }

                // This event matches this Serie and will be kept
                String dateConverted = format.format(elem.getTimestamp());
                EventData ed = new EventData(curser.getId(), dateConverted, elem.getValue().toString());
                dbc.addResult(ed);
                // continue because an event may match several series
            }
        }

        // replay from database completed
        dbc.setReplayCompleted();
    }

}
