/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: FileExtractor.java 7834 2011-03-17 15:13:37Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.service;

import org.ow2.jasmine.monitoring.eos.monitoring.EventData;
import org.ow2.jasmine.monitoring.eos.monitoring.Serie;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.*;

/**
 * Extractor for File with no header (usually produced by eventswitch)
 * @author durieuxp
 */
public class FileExtractor extends Extractor {

    private BufferedReader buffer;

    // ------------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------------

    /**
     * Constructor for DashboardDataCollector
     */
    public FileExtractor(DashboardDataCollector dbc, BufferedReader buffer, Date from, Date to) {
        super(dbc, from, to);
        this.buffer = buffer;
    }

    // ------------------------------------------------------------------------
    // Thread implementation
    // ------------------------------------------------------------------------

    /**
     * Run method called when the thread is started.
     */
    public void run() {

        // parse the file
        String line = "";
        try {
            while ((line = buffer.readLine()) != null) {
                //logger.debug("Reading line:" + line);
                // split the line just once
                String[] split = line.split(";");
                if (split.length <= 6) {
                    // Ignore all lines incomplete.
                    // This may happen in case of "poll -slb" for example.
                    //logger.debug("line incomplete");
                    continue;
                }
                String date = split[1];
                String server = split[4];

                // compute mbean and attribute
                String mbeanattr = split[5];
                int index = mbeanattr.lastIndexOf(":");
                String mbean = mbeanattr.substring(0, index);
                String attr = mbeanattr.substring(index + 1);

                String value = split[6];

                // Find series accepting this event.
                for (Serie curser : dbc.getSeries()) {
                    // Check Attribute
                    if (!attr.equals(curser.getYcol())) {
                        //logger.debug("bad attr:" + curser.getYcol());
                        continue;
                    }

                    // Check pattern
                    if (! dbc.match(mbean, curser.getPattern())) {
                        continue;
                    }

                    // Check server
                    if (curser.getServer() != null && curser.getServer().length() > 0 && !curser.getServer().equals(server)) {
                        //logger.debug("bad server:" + curser.getServer());
                        continue;
                    }

                    // This event matches this Serie and will be kept
                    EventData ed = new EventData(curser.getId(), date, value);
                    dbc.addResult(ed);
                    // continue because an event may match several series
                }
            }
        } catch (IOException e) {
            logger.error("Error reading file: " + e);
        } finally {
            dbc.setReplayCompleted();
        }
    }
}
