/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: HeaderFileExtractor.java 7834 2011-03-17 15:13:37Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.service;

import org.ow2.jasmine.monitoring.eos.monitoring.EventData;
import org.ow2.jasmine.monitoring.eos.monitoring.Serie;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Extractor for File with header (usually produced by mbeancmd)
 * @author durieuxp
 */
public class HeaderFileExtractor extends Extractor {

    private String header;

    private BufferedReader buffer;

    private int dateind;

    // ------------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------------

    /**
     * Constructor for DashboardDataCollector
     */
    public HeaderFileExtractor(DashboardDataCollector dbc, String header, int dateind, BufferedReader buffer, Date from, Date to) {
        super(dbc, from, to);
        this.header = header;
        this.buffer = buffer;
        this.dateind = dateind;
    }

    // ------------------------------------------------------------------------
    // Thread implementation
    // ------------------------------------------------------------------------

    /**
     * Run method called when the thread is started.
     */
    public void run() {

        // Parse the header to look for column names
        String[] headerArray = header.split(";");
        int mbeanCol = -1;
        int serverCol = -1;
        Map<String, Integer> valueCol = new HashMap<String, Integer>();
        for (int index = 0; index < headerArray.length; index++) {
            if (headerArray[index].equals("server")) {
                serverCol = index;
            }
            if (headerArray[index].equals("mbean")) {
                mbeanCol = index;
            }
            for (Serie afil : dbc.getSeries()) {
                if (afil.getYcol().equals(headerArray[index])) {
                    valueCol.put(afil.getId(), new Integer(index));
                }
            }
        }
        if (serverCol == -1) {
            logger.error("Bad log file: No server in header");
        }
        if (mbeanCol == -1) {
            logger.error("Bad log file: No mbean in header");
        }


        // parse the file
        String line = "";
        try {
            while ((line = buffer.readLine()) != null) {
                // split the line just once
                String[] split = line.split(";");
                if (split.length < headerArray.length) {
                    // Ignore all lines incomplete.
                    // This may happen in case of "poll -slb" for example.
                    continue;
                }
                String date = split[dateind];
                String server = split[serverCol];

                // compute mbean
                String mbeanattr = split[mbeanCol];
                int index = mbeanattr.lastIndexOf(":");
                String mbean = mbeanattr.substring(0, index);

                // Find series accepting this event.
                for (Serie curser : dbc.getSeries()) {
                    Integer valueIndex = valueCol.get(curser.getId());
                    if (valueIndex == null) {
                        // This serie is not present in this file.
                        continue;
                    }
                    String value = split[valueIndex.intValue()];

                    // Check pattern
                    if (! dbc.match(mbean, curser.getPattern())) {
                        continue;
                    }

                    // Check server
                    if (curser.getServer() != null && curser.getServer().length() > 0 && !curser.getServer().equals(server)) {
                        continue;
                    }

                    // This event matches this Serie and will be kept
                    EventData ed = new EventData(curser.getId(), date, value);
                    dbc.addResult(ed);
                    // continue because an event may match several series
                }
            }
        } catch (IOException e) {
            logger.error("Error reading file: " + e);
        } finally {
            dbc.setReplayCompleted();
        }

    }
}
